// Copyright (c) 1995, 1996 Regents of the University of California.
// All rights reserved.
//
// This software was developed by the Arcadia project
// at the University of California, Irvine.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the University of California, Irvine.  The name of the
// University may not be used to endorse or promote products derived
// from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.

// File: ModeCreateArc.java
// Classes: ModeCreateArc
// Original Author: ics125b spring 1996
// $Id: ModeCreateArc.java,v 1.12 1997/01/17 00:21:24 jrobbins Exp $

package uci.graphedit;

import java.awt.*;

/** A Mode to interpret user input while creating an arc. Basically
 * mouse down starts creating an arc from a source NetPort, mouse
 * motion draws a rubberband line, mouse up finds the destination port
 * and finishes creating the arc and makes an ArcPerspective and sends
 * it to the back of the Layer.  */
public class ModeCreateArc extends ModeCreate {

  /** The NetPort where the arc is drawn from */
  protected NetPort startPort;
  /** The Fig that presents the starting NetPort */
  protected Fig startPortFig;
  /** The Perspective on the NetNode that owns the start port */
  protected Perspective sourcePerspective;
  /** The new NetArc that is being created */
  protected NetArc newArc;

  /** Construct a new ModeCreateArc */
  public ModeCreateArc(Editor par) { super(par); }
  
  public String instructions() {
    return "Drag to define an arc to another port";
  }

  
  /** Create the new item that will be drawn. In this case I would
   * rather create the ArcPerspective when I am done. Here I just
   * create a rubberband line to show during the interaction. */
  public DiagramElement createNewItem(Event e) {
    return new FigLine(e.x, e.y, 0, 0, Globals.prefs().rubberbandAttrs());    
  }
  
  /** On mouse down determine what port the user is dragging from. The
   * mouse down event is passed in from ModeSelect. */
  public boolean mouseDown(Event e, int x, int y) {
    DiagramElement underMouse = parent.pick(x, y);
    if (underMouse == null) { done(); return true; }
    if (!(underMouse instanceof Perspective)) { done(); return true; }
    sourcePerspective = (Perspective) underMouse;
    startPort = sourcePerspective.pickPort(x, y);
    if (startPort == null) { done(); return true; }
    startPortFig = sourcePerspective.getPortFig(startPort);
    return super.mouseDown(e, x, y);
  }   
  
  /** On mouse up, find the destination port, ask the NetArc
   * (SampleArc) to connect the two ports. If that connection is
   * allowed, then construct a new ArcPerspective and add it to the
   * Layer and send it to the back. */
  public boolean mouseUp(Event e, int x, int y) {     
    Class arcClass;
    
    DiagramElement de = parent.pick(x, y);
    
    if (de != null) { 
      if (de instanceof Perspective) {
	Perspective destPerspective = (Perspective) de;
	/* If its a perspective, then check within the  */
	/* perspective to see if a port exists */
	NetPort foundPort = destPerspective.pickPort(x, y);
	
	if (foundPort != null && foundPort != startPort) {
	  Fig destPortFig;
	  destPortFig = destPerspective.getPortFig(foundPort);
	  try { arcClass = Class.forName("uci.graphedit.SampleArc"); }
	  // Needs-More-Work: what subclass of NetArc?
	  catch (java.lang.ClassNotFoundException ignore) { return false; }

	  try { newArc = (NetArc) arcClass.newInstance(); }
	  catch (java.lang.IllegalAccessException ignore) { return false; }
	  catch (java.lang.InstantiationException ignore) { return false; }
	  
	  if (!newArc.connect(startPort, foundPort)) {
	    newItem.damagedIn(parent);
	    newItem = null;
	    done();
	    return true;
	  }
	  ArcPerspective ap;
	  ap = new ArcPerspective(startPortFig, destPortFig,
				  sourcePerspective, destPerspective,
				  newArc);
	  parent.add(ap);
	  ap.reorder(ActionReorder.SEND_TO_BACK, parent.view());
	  parent.selectItem(ap);
	}
      }
    }
    newItem.damagedIn(parent);
    newItem = null;
    done();
    return true;
  }
  
} /* end class ModeCreateArc */

